﻿// 
// Copyright (c) 2013-2016 Carsten Sonne Larsen <cs@innolan.dk>
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System;
using System.Collections.Generic;
using System.Linq;

namespace Ntp.Common.Web
{
    public static class UriExtensions
    {
        public static Uri Append(this Uri uri, params string[] paths)
        {
            string basePath = uri.ToString();
            var uriKind = Uri.IsWellFormedUriString(basePath, UriKind.Absolute)
                ? UriKind.Absolute
                : UriKind.Relative;

            string resPath = paths.Aggregate(
                basePath,
                (current, path) => $"{current.TrimEnd('/')}/{path.TrimStart('/')}"
                );

            return new Uri(resPath, uriKind);
        }

        public static Uri Append(this Uri uri, Uri relativeUri, params string[] paths)
        {
            var list = new List<string> {relativeUri.ToString()};
            list.AddRange(paths);
            return uri.Append(list.ToArray());
        }

        public static Uri Append(this Uri uri, Uri relativeUri, string path)
        {
            var list = new List<string> {relativeUri.ToString(), path};
            return uri.Append(list.ToArray());
        }

        public static Uri AppendExtension(this Uri uri, string extension)
        {
            string basePath = uri.ToString().TrimEnd('/') + extension;
            return new Uri(basePath, uri.IsAbsoluteUri ? UriKind.Absolute : UriKind.Relative);
        }

        public static string ToHtmlString(this Uri uri)
        {
            return Uri.EscapeUriString(uri.ToString());
        }
    }
}